@@ -1,10 +1,15 @@
 package Reddit::Client;
 
-our $VERSION = '1.201';
+our $VERSION = '1.231';
+# be pretty amazed if we didn't break anything 
+# 1.23 get_modmail_raw
+#	   get_refresh_token
+# 1.222 Removing unused fields from Comment/Post
+# 1.22 get refresh token functionality
+# 1.21 added spam()
+# 1.21 get modmail. hidden feature because it's a godawful mess
+# 1.202 username is now required for get or create multis unless script auth
 # 1.20 web type apps now supported!
-# get_wiki, get_wiki_data
-## edit_wiki now takes alias 'sub'
-# dropped token refresh time to 3595 to account for lag or other fuckery
 
 $VERSION = eval $VERSION;
 
@@ -36,7 +41,7 @@ use constant VIEW_HOT                => '';
 use constant VIEW_NEW                => 'new';
 use constant VIEW_CONTROVERSIAL      => 'controversial';
 use constant VIEW_TOP                => 'top';
-use constant VIEW_RISING	     => 'rising';
+use constant VIEW_RISING	          => 'rising';
 use constant VIEW_DEFAULT            => VIEW_HOT;
 
 use constant VOTE_UP                 => 1;
@@ -110,6 +115,7 @@ use constant API_REMOVE              => 34;
 use constant API_APPROVE             => 35;
 use constant API_IGNORE_REPORTS      => 36;
 use constant API_GETWIKI             => 37;
+use constant API_GET_MODMAIL         => 38;
 
 #===============================================================================
 # Parameters
@@ -161,55 +167,27 @@ $API[API_EDIT          ] = ['POST', '/api/editusertext'       ];
 $API[API_REMOVE        ] = ['POST', '/api/remove'             ];
 $API[API_APPROVE       ] = ['POST', '/api/approve'            ];
 $API[API_IGNORE_REPORTS] = ['POST', '/api/ignore_reports'     ];
-#===============================================================================
-# Package routines
-#===============================================================================
-
-sub DEBUG {
-    if ($DEBUG) {
-        my ($format, @args) = @_;
-        my $ts  = strftime "%Y-%m-%d %H:%M:%S", localtime;
-        my $msg = sprintf $format, @args;
-        chomp $msg;
-        printf STDERR "[%s] [ %s ]\n", $ts, $msg;
-    }
-}
-
-sub subreddit {
-    my $subject = shift;
-    $subject =~ s/^\/r//; # trim leading /r
-    $subject =~ s/^\///;  # trim leading slashes
-    $subject =~ s/\/$//;  # trim trailing slashes
-
-    if ($subject !~ /\//) {   # no slashes in name - it's probably good
-        if ($subject eq '') { # front page
-            return '';
-        } else {              # subreddit
-            return $subject;
-        }
-    } else { # fail
-        return;
-    }
-}
+$API[API_GET_MODMAIL   ] = ['GET',  '/api/mod/conversations'  ];
 
 #===============================================================================
 # Class methods
 #===============================================================================
 
 use fields (
-    	'modhash',      # store session modhash
-    	'cookie',       # store user cookie
-    	'session_file', # path to session file
-    	'user_agent',   # user agent string
-    	'token',	# oauth authorization token
-    	'tokentype',    # unused but saved for reference
-    	'last_token',	# time last token was acquired
-    	'client_id',	# These 4 values saved for automatic token refreshing
-    	'secret',       
+    'modhash',      # store session modhash
+    'cookie',       # store user cookie
+    'session_file', # path to session file
+    'user_agent',   # user agent string
+    'token',	# oauth authorization token
+    'tokentype',    # unused but saved for reference
+    'last_token',	# time last token was acquired
+    'client_id',	# These 4 values saved for automatic token refreshing
+    'secret',       
 	'username',
 	'password',
 	'request_errors',# print request errors
 	'print_response',# print entire response
+	'print_request', # print entire request
 	'refresh_token', # oauth refresh token
 	'auth_type',
 );
@@ -224,6 +202,7 @@ sub new {
 	$self->{user_agent} 	= $param{user_agent};
 	$self->{request_errors} = $param{request_errors} || 0;
 	$self->{print_response} = $param{print_response} || 0;
+	$self->{print_request}  = $param{print_request}  || 0;
 
 	if ($param{username} or $param{password}) {
 		if (!$param{username} or !$param{password}) {
@@ -249,8 +228,12 @@ sub new {
 
 		$self->get_token();
 	} else {
+		# optionall allow people to pass in client id and secret now, for people
+		# who choose to get refresh token from an RC object
+		$self->{client_id}	= $param{client_id} if $param{client_id};
+		$self->{secret} 	= $param{secret} if $param{secret};
 		# can this even be run without auth anymore?
-		$self->{auth_type} = 'none';
+		$self->{auth_type}  = 'none';
 	}
 
     return $self;
@@ -262,7 +245,7 @@ sub version {
 }
 
 #===============================================================================
-# Internal management
+# Requests and Oauth
 #===============================================================================
 
 sub request {
@@ -293,6 +276,7 @@ sub request {
 	tokentype  => $self->{tokentype},
 	request_errors=> $self->{request_errors},
 	print_response=> $self->{print_response},
+	print_request=>  $self->{print_request},
     );
 
     return $request->send;
@@ -312,11 +296,9 @@ sub get_token {
 		$self->{password}	= $param{password};
 
 	} elsif ($param{refresh_token}) {
-		die "get_token: if refresh_token is provided, all 3 webapp-type authenticaltion arguments (refresh_token, client_id, secret) are required." unless $param{refresh_token} and $param{client_id} and $param{secret};
-
 		$self->{auth_type} 	= 'webapp';
-		$self->{client_id}	= $param{client_id};
-		$self->{secret} 	= $param{secret};
+		$self->{client_id}	= $param{client_id} || $self->{client_id} || die "get_token: 'client_id' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object. ";
+		$self->{secret} 	= $param{secret} || $self->{secret} || die "get_token: 'secret' must be set, either as a parameter to get_token or when instantiating the Reddit::Client object. ";
 		$self->{refresh_token} 	= $param{refresh_token};
 	}
 
@@ -338,7 +320,6 @@ sub get_token {
 	} else { die "get_token: invalid auth type"; }
 
 	my $message = Reddit::Client::Request->token_request(%p);
-	#dd $message;
 	my $j = decode_json($message);
 	$self->{token} 		= $j->{access_token};
 	$self->{tokentype} 	= $j->{token_type};
@@ -350,6 +331,30 @@ sub has_token {
 	my $self = shift;
 	return (!$self->{last_token} || $self->{last_token} <= time - 3595) ? 0 : 1;
 }
+# 
+# This must be called in static context because no refresh token or user/
+# pass combination exist. We would have to add a third flow and that doesn't
+# seem worth it.
+#
+# We could call it in an empty RC object, but that would require all sorts
+# of annoyoing conditions, and all other methods would be broken until
+# tokens were obtained
+sub get_refresh_token {
+	my ($self, %param) = @_;
+
+	my %data;
+	$data{code}			= $param{code} || die "'code' is required.\n";
+	$data{redirect_uri}	= $param{redirect_uri} || die "'redirect_uri' is required.\n";
+	$data{client_id}	= (ref $self eq 'HASH' and $self->{client_id}  ? $self->{client_id} : undef) || $param{client_id}   || die "'client_id' is required.\n";
+	$data{secret}		= (ref $self eq 'HASH' and $self->{secret}     ? $self->{secret} : undef)    || $param{secret}      || die "'secret' is required.";
+	$data{ua}			= (ref $self eq 'HASH' and $self->{user_agent} ? $self->{user_agent} : undef) || $param{user_agent} || die "'user_agent' is required.";
+	#$data{ua}			= $param{user_agent} || die "user_agent is required";
+	$data{grant_type} 	= 'authorization_code';
+	$data{duration}		= 'permanent';
+
+	my $refresh_token = Reddit::Client::Request->refresh_token_request(%data);
+	return $refresh_token;
+}
 
 sub json_request {
     my ($self, $method, $path, $query, $post_data) = @_;
@@ -597,7 +602,6 @@ sub search {
         	map {Reddit::Client::Link->new($self, $_->{data})} @{$result->{data}{children}}
     	       ];
 }
-
 sub get_permalink {
 	# This still makes an extra request. Why?
 	my ($self, $commentid, $post_fullname) = @_;
@@ -649,7 +653,7 @@ sub fetch_links {
 }
 
 sub get_links { # alias for fetch_links to make naming convention consistent
-    	my ($self, %param) = @_;
+    my ($self, %param) = @_;
 	return $self->fetch_links(%param);
 }
 
@@ -696,6 +700,7 @@ sub get_subreddit_comments {
         	data     => $query,
     	);
 
+		#return $result->{data}{children}[0]->{data};
     	return [
         	 map {Reddit::Client::Comment->new($self, $_->{data})} @{$result->{data}{children}} 
     	];
@@ -710,7 +715,18 @@ sub remove {
 	
     	my $result = $self->api_json_request(
 		api  => API_REMOVE,
-		data => { id => $fullname },
+		data => { id => $fullname, spam=> 'false' },
+	);
+	return $result;
+}
+# like remove, but sets spam flag
+sub spam {
+	my $self = shift;
+	my $fullname = shift || croak "arg 1 (fullname) is required.";
+	
+    	my $result = $self->api_json_request(
+		api  => API_REMOVE,
+		data => { id => $fullname, spam => 'true' },
 	);
 	return $result;
 }
@@ -734,7 +750,7 @@ sub ignore_reports {
 	);
 	return $result;
 }
-# reports spam modqueue unmoderated edited
+# modqueue 
 sub get_modlinks {
     	my ($self, %param) = @_;
 
@@ -762,7 +778,56 @@ sub get_modlinks {
 	];
 }
 
-# Get information about a user
+# after: conversation id
+# entity: comma-delimited list of subreddit names
+# limit
+# sort: one of (recent, mod, user, unread)
+# state: one of (new, inprogress, mod, notifications, archived, highlighted, all)
+sub get_modmail {
+    	my ($self, %param) = @_;
+
+	my $data	= {};
+	$data->{sort}	= $param{sort} || 'unread';
+	$data->{state}	= $param{state} || 'all';
+	# What is the difference between sort 'unread' and state 'new'? Possibly state 'new' is unseen by any mod?
+	$data->{after}	= $param{after} if $param{after};
+	$data->{limit}	= exists $param{limit} ? ( $param{limit} ? $param{limit} : 500 )  : DEFAULT_LIMIT;
+
+	my $subs	= $param{entity} || $param{subreddits} || $param{subs};
+	if ($subs) {
+		$subs		= join ",", @$subs if ref $subs eq 'ARRAY';	
+		$data->{entity} = $subs if $subs;
+	}
+	my $result = $self->api_json_request(
+		api	=> API_GET_MODMAIL,
+		data	=> $data,
+	);
+	return $result;
+}
+sub get_modmail_raw {
+    	my ($self, %param) = @_;
+
+	my $data	= {};
+	$data->{sort}	= $param{sort} || 'unread';
+	$data->{state}	= $param{state} || 'all';
+	$data->{after}	= $param{after} if $param{after};
+	$data->{limit}	= exists $param{limit} ? ( $param{limit} ? $param{limit} : 500 )  : DEFAULT_LIMIT;
+
+	my $subs	= $param{entity} || $param{subreddits} || $param{subs};
+	if ($subs) {
+		$subs		= join ",", @$subs if ref $subs eq 'ARRAY';	
+		$data->{entity} = $subs if $subs;
+	}
+	my $result = $self->api_json_request(
+		api	=> API_GET_MODMAIL,
+		data	=> $data,
+	);
+	return $result;
+}
+
+#=============================================================
+# Users
+#=============================================================
 sub get_user {
     	my ($self, %param) = @_;
 	my $view	= $param{view} || 'overview';
@@ -864,8 +929,8 @@ sub submit_link {
         url         => $url,
         sr          => $subreddit,
         kind        => SUBMIT_LINK,
-	sendreplies => $replies,
-	resubmit    => $repost,
+		sendreplies => $replies,
+		resubmit    => $repost,
     });
 
     return $result->{data}{name};
@@ -1042,6 +1107,8 @@ sub submit_comment {
     my ($self, %param) = @_;
     my $parent_id = $param{parent} || $param{parent_id} || croak 'Expected "parent"';
     my $comment   = $param{text}      || croak 'Expected "text"';
+	# the replies option, it does nothing
+    #my $replies = exists $param{inbox_replies} ? ($param{inbox_replies} ? "true" : "false") : "true";
 
     croak '$fullname must be a post or comment' if !$self->ispost($parent_id) && !$self->iscomment($parent_id);
     DEBUG('Submit comment under %s', $parent_id);
@@ -1049,6 +1116,7 @@ sub submit_comment {
     my $result = $self->api_json_request(api => API_COMMENT, data => {
         thing_id => $parent_id,
         text     => $comment,
+		#sendreplies=>$replies,
     });
 
     return $result->{data}{things}[0]{data}{id};
@@ -1147,6 +1215,7 @@ sub create_multi {
         my ($self, %param) = @_;
 	my $data 	= {};
         my $model 	= {};
+	my $username    = $param{username} || $self->{username} || die "'username' is required.";
 
         $model->{display_name}  = $param{name} || croak "Expected 'name'.";
         if (length($model->{display_name}) > 50) { croak "max length of 'name' is 50."; }
@@ -1192,11 +1261,11 @@ sub create_multi {
 
 	# Put a ribbon on it
 	$data->{model} = JSON::encode_json($model); 
-	$data->{multipath} = "/user/$self->{username}/m/$model->{display_name}";
+	$data->{multipath} = "/user/$username/m/$model->{display_name}";
 
 	my $result = $self->api_json_request( 
 		api => $param{edit} ? API_EDITMULTI : API_CREATEMULTI, 
-		args => [$self->{username}, $model->{display_name}],
+		args => [$username, $model->{display_name}],
 		data => $data,
 	);
 
@@ -1251,6 +1320,32 @@ sub get_type {
 	my ($self, $name) = @_;
     	return lc substr $name, 0, 2;
 }
+sub DEBUG {
+    if ($DEBUG) {
+        my ($format, @args) = @_;
+        my $ts  = strftime "%Y-%m-%d %H:%M:%S", localtime;
+        my $msg = sprintf $format, @args;
+        chomp $msg;
+        printf STDERR "[%s] [ %s ]\n", $ts, $msg;
+    }
+}
+
+sub subreddit {
+    my $subject = shift;
+    $subject =~ s/^\/r//; # trim leading /r
+    $subject =~ s/^\///;  # trim leading slashes
+    $subject =~ s/\/$//;  # trim trailing slashes
+
+    if ($subject !~ /\//) {   # no slashes in name - it's probably good
+        if ($subject eq '') { # front page
+            return '';
+        } else {              # subreddit
+            return $subject;
+        }
+    } else { # fail
+        return;
+    }
+}
 
 1;
 
@@ -1262,7 +1357,7 @@ __END__
 
 Reddit::Client - A Perl wrapper for the Reddit API.
 
-Up to date, nicely-formatted documentation can be found at L<http://redditclient.readthedocs.org/en/latest/>.  What follows is probably not incorrect, but is probably not complete or current either.
+Up-to-date, nicely-formatted documentation can be found at L<http://redditclient.readthedocs.io>.  What follows is probably not incorrect, but is almost definitely not complete or current.
 
 =head1 SYNOPSIS
 
@@ -1276,7 +1371,7 @@ Up to date, nicely-formatted documentation can be found at L<http://redditclient
 
     # Create a Reddit::Client object and authorize in one step
     my $reddit = new Reddit::Client(
-	user_agent 	=> 'MyScriptName 1.0 by /u/earth-tone',
+	user_agent 	=> 'MyScriptName 1.0 by /u/myusername',
 	client_id	=> $client_id,
 	secret		=> $secret,
 	username	=> $username,
@@ -1286,7 +1381,7 @@ Up to date, nicely-formatted documentation can be found at L<http://redditclient
     # Or create object then authorize.
     # Useful if you need to switch between accounts, for example if you were to check the inboxes of several accounts.
     my $reddit = Reddit::Client->new(
-        user_agent   	=> 'MyApp/1.0 by /u/earth-tone',
+        user_agent   	=> 'MyApp/1.0 by /u/myusername',
     );
 
     $reddit->get_token(
