@@ -4,11 +4,12 @@ use strict;
 use warnings;
 use Carp;
 
+use JSON           qw/decode_json/;
 use LWP::UserAgent qw//;
 use HTTP::Request  qw//;
 use URI::Encode    qw/uri_encode/;
 use URI::Escape    qw/uri_escape/; # next update, also line 122
-#use Data::Dump::Color;
+use Data::Dumper;
 
 require Reddit::Client;
 
@@ -24,6 +25,7 @@ use fields (
     'tokentype',
 	'request_errors',
 	'print_response',
+	'print_request',
 );
 
 sub new {
@@ -35,10 +37,11 @@ sub new {
     $self->{post_data}  = $param{post_data};
     $self->{cookie}     = $param{cookie};
     $self->{modhash}    = $param{modhash};
-    $self->{token}	= $param{token};
+    $self->{token}		= $param{token};
     $self->{tokentype}	= $param{tokentype};
     $self->{request_errors} = $param{request_errors} || 0;
     $self->{print_response} = $param{print_response} || 0;
+    $self->{print_request}  = $param{print_request} || 0;
 
     if (defined $self->{query}) {
         ref $self->{query} eq 'HASH' || croak 'Expected HASH ref for "query"';
@@ -102,15 +105,27 @@ sub send {
     my $ua  = LWP::UserAgent->new(agent => $self->{user_agent}, env_proxy => 1);
     my $res = $ua->request($request);
 
+	if ($self->{print_request}) {
+		print Dumper($request);
+		print Dumper($res);
+	} elsif ($self->{print_response}) {
+		print $res->content . "\n";
+	}
+
     if ($res->is_success) {
-	print $res->content . "\n" if $self->{print_response};
         return $res->content;
     } else {
-	if ($self->{request_errors}) {
-        	croak "Request error: HTTP ".$res->status_line .", Content: $res->{_content}";
-	} else {
-		croak sprintf('Request error: HTTP %s', $res->status_line);
-	}
+		if ($self->{request_errors}) {
+			croak "Request error: HTTP ".$res->status_line .", Content: $res->{_content}";
+		} else {
+			croak sprintf("Request error: HTTP %s\n", $res->status_line);
+		}
+
+		#print "-- test -- \n";
+		#print Dumper($request);
+		#print Dumper($res);
+		# print_request overrides print_response since it contains the response
+		#die;	
     }
 }
 
@@ -119,7 +134,7 @@ sub token_request {
 
 	my $url = "https://$param{client_id}:$param{secret}\@www.reddit.com/api/v1/access_token";
 
-    	my $ua = LWP::UserAgent->new(agent => $param{user_agent});
+    my $ua = LWP::UserAgent->new(agent => $param{user_agent});
 	my $req = HTTP::Request->new(POST => $url);
 	$req->header('content-type' => 'application/x-www-form-urlencoded');
 
@@ -144,6 +159,54 @@ sub token_request {
     	}
 }
 
+sub refresh_token_request {
+	my ($self, %data)	= @_;
+
+    # create user agent
+    my $ua      = new LWP::UserAgent( agent=> $data{ua} );
+    # create new request
+    my $request = new HTTP::Request();
+    # set the request method
+    $request->method("POST");
+    # set request url
+    my $url = "https://$data{client_id}:$data{secret}\@www.reddit.com/api/v1/access_token";
+    $request->uri($url);
+
+	my $reqdata = {
+        grant_type  => 'authorization_code',
+        code        => $data{code},
+        redirect_uri=> $data{redirect_uri},
+        duration    => 'permanent',
+    };
+
+    $request->content_type('application/x-www-form-urlencoded');
+
+    my $opt   = { encode_reserved => 1 };
+    my $encoded = join '&', map { uri_encode($_, $opt) . '=' . uri_encode($reqdata->{$_}, $opt) } sort keys %$reqdata;
+
+    $request->content($encoded);
+
+    my $result = $ua->request($request);
+
+    if ($result->is_success) {
+        my $j = decode_json $result->content;
+        my $tok = $j->{refresh_token};
+
+		return $tok;
+    } else {
+        print "refresh_token_request: something went wrong\n";
+		print $result->status_line;
+		print $result->{_content};
+
+		if ($data{request_errors}) {
+			print Dumper($request);
+			print Dumper($result);
+		}
+		die;
+    }
+
+}
+
 1;
 
 __END__
